﻿using System.Net;
using Amazon.S3;
using Amazon.S3.Transfer;
using Amazon.TranscribeService;
using Amazon.TranscribeService.Model;

string bucketName = "your-bucket-name";
string fileToTranscribe = "your-audio-file.mp3";

IAmazonTranscribeService amazonTranscribeService = new AmazonTranscribeServiceClient();
TransferUtility transferUtility = new TransferUtility(new AmazonS3Client());

await transferUtility.UploadAsync(fileToTranscribe, bucketName);
string mp3S3Location = $"s3://{bucketName}/{fileToTranscribe}";
string jobName = Guid.NewGuid().ToString();
var startTranscriptionJobResponse = await StartTranscriptionJob(mp3S3Location, jobName);
Console.WriteLine($"Status of start transcription job request: {startTranscriptionJobResponse}");

if (await PollTranscriptionJob(jobName))
{
    await transferUtility.DownloadAsync($"{jobName}.json", bucketName, $"{jobName}.json");
}
else 
{
    Console.WriteLine("Transcription job failed");
}

async Task<HttpStatusCode> StartTranscriptionJob(string s3Uri, string transcriptionJobName)
{
    var startTranscriptionJobRequest = new StartTranscriptionJobRequest()
    {
        TranscriptionJobName = transcriptionJobName,
        LanguageCode = LanguageCode.EnUS,
        Media = new Media()
        {
            MediaFileUri = s3Uri
        },
        OutputBucketName = bucketName
    };

    var startTranscriptionJobResponse = await amazonTranscribeService.StartTranscriptionJobAsync(startTranscriptionJobRequest);
    return startTranscriptionJobResponse.HttpStatusCode;
}

async Task<bool> PollTranscriptionJob(string jobName)
{
    while (true)
    {
        var getTranscriptionJobResponse = await amazonTranscribeService.GetTranscriptionJobAsync(new GetTranscriptionJobRequest()
        {
            TranscriptionJobName = jobName
        });

        var transcriptionJobStatus = getTranscriptionJobResponse.TranscriptionJob.TranscriptionJobStatus;
        if (transcriptionJobStatus == TranscriptionJobStatus.COMPLETED)
        {
            Console.WriteLine("Transcription job completed");
            Console.WriteLine($"Output is available at {getTranscriptionJobResponse.TranscriptionJob.Transcript.TranscriptFileUri}");
            return true;
        }
        else if (transcriptionJobStatus == TranscriptionJobStatus.FAILED)
        {
            Console.WriteLine("Transcription job failed");
            return false;
        }
        else
        {
            Console.WriteLine("Transcription job not completed yet");
        }
        await Task.Delay(5000);
    }
}